@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,6 +50,7 @@
 #include <qaudioformat.h>
 #include <QtCore/qelapsedtimer.h>
 #include <QtMultimedia/qvideosurfaceformat.h>
+#include <private/qmultimediautils_p.h>
 
 #include <gst/audio/audio.h>
 #include <gst/video/video.h>
@@ -479,7 +486,7 @@ QMultimedia::SupportEstimate QGstUtils::hasSupport(const QString &mimeType,
     }
 
     int supportedCodecCount = 0;
-    foreach (const QString &codec, codecs) {
+    for (const QString &codec : codecs) {
         QString codecLowcase = codec.toLower();
         const char* codecAlias = getCodecAlias(codecLowcase);
         if (codecAlias) {
@@ -582,10 +589,10 @@ QVector<QGstUtils::CameraInfo> QGstUtils::enumerateCameras(GstElementFactory *fa
     QDir devDir(QStringLiteral("/dev"));
     devDir.setFilter(QDir::System);
 
-    QFileInfoList entries = devDir.entryInfoList(QStringList()
+    const QFileInfoList entries = devDir.entryInfoList(QStringList()
                 << QStringLiteral("video*"));
 
-    foreach (const QFileInfo &entryInfo, entries) {
+    for (const QFileInfo &entryInfo : entries) {
         //qDebug() << "Try" << entryInfo.filePath();
 
         int fd = qt_safe_open(entryInfo.filePath().toLatin1().constData(), O_RDWR );
@@ -642,7 +649,9 @@ QList<QByteArray> QGstUtils::cameraDevices(GstElementFactory * factory)
 {
     QList<QByteArray> devices;
 
-    foreach (const CameraInfo &camera, enumerateCameras(factory))
+    const auto cameras = enumerateCameras(factory);
+    devices.reserve(cameras.size());
+    for (const CameraInfo &camera : cameras)
         devices.append(camera.name.toUtf8());
 
     return devices;
@@ -650,7 +659,8 @@ QList<QByteArray> QGstUtils::cameraDevices(GstElementFactory * factory)
 
 QString QGstUtils::cameraDescription(const QString &device, GstElementFactory * factory)
 {
-    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+    const auto cameras = enumerateCameras(factory);
+    for (const CameraInfo &camera : cameras) {
         if (camera.name == device)
             return camera.description;
     }
@@ -659,7 +669,8 @@ QString QGstUtils::cameraDescription(const QString &device, GstElementFactory *
 
 QCamera::Position QGstUtils::cameraPosition(const QString &device, GstElementFactory * factory)
 {
-    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+    const auto cameras = enumerateCameras(factory);
+    for (const CameraInfo &camera : cameras) {
         if (camera.name == device)
             return camera.position;
     }
@@ -668,7 +679,8 @@ QCamera::Position QGstUtils::cameraPosition(const QString &device, GstElementFac
 
 int QGstUtils::cameraOrientation(const QString &device, GstElementFactory * factory)
 {
-    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+    const auto cameras = enumerateCameras(factory);
+    for (const CameraInfo &camera : cameras) {
         if (camera.name == device)
             return camera.orientation;
     }
@@ -677,7 +689,8 @@ int QGstUtils::cameraOrientation(const QString &device, GstElementFactory * fact
 
 QByteArray QGstUtils::cameraDriver(const QString &device, GstElementFactory *factory)
 {
-    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+    const auto cameras = enumerateCameras(factory);
+    for (const CameraInfo &camera : cameras) {
         if (camera.name == device)
             return camera.driver;
     }
@@ -748,8 +761,8 @@ QSet<QString> QGstUtils::supportedMimeTypes(bool (*isValidFactory)(GstElementFac
                             if (value) {
                                 gchar *str = gst_value_serialize(value);
                                 QString versions(str);
-                                QStringList elements = versions.split(QRegExp("\\D+"), QString::SkipEmptyParts);
-                                foreach (const QString &e, elements)
+                                const QStringList elements = versions.split(QRegExp("\\D+"), QString::SkipEmptyParts);
+                                for (const QString &e : elements)
                                     supportedMimeTypes.insert(nameLowcase + e);
                                 g_free(str);
                             }
@@ -767,13 +780,14 @@ QSet<QString> QGstUtils::supportedMimeTypes(bool (*isValidFactory)(GstElementFac
     QStringList list = supportedMimeTypes.toList();
     list.sort();
     if (qgetenv("QT_DEBUG_PLUGINS").toInt() > 0) {
-        foreach (const QString &type, list)
+        for (const QString &type : qAsConst(list))
             qDebug() << type;
     }
 #endif
     return supportedMimeTypes;
 }
 
+#if GST_CHECK_VERSION(1, 0, 0)
 namespace {
 
 struct ColorFormat { QImage::Format imageFormat; GstVideoFormat gstFormat; };
@@ -786,6 +800,7 @@ static const ColorFormat qt_colorLookup[] =
 };
 
 }
+#endif
 
 #if GST_CHECK_VERSION(1,0,0)
 QImage QGstUtils::bufferToImage(GstBuffer *buffer, const GstVideoInfo &videoInfo)
@@ -1107,7 +1122,7 @@ GstCaps *QGstUtils::capsForFormats(const QList<QVideoFrame::PixelFormat> &format
     GstCaps *caps = gst_caps_new_empty();
 
 #if GST_CHECK_VERSION(1,0,0)
-    foreach (QVideoFrame::PixelFormat format, formats) {
+    for (QVideoFrame::PixelFormat format : formats) {
         int index = indexOfVideoFormat(format);
 
         if (index != -1) {
@@ -1118,7 +1133,7 @@ GstCaps *QGstUtils::capsForFormats(const QList<QVideoFrame::PixelFormat> &format
         }
     }
 #else
-    foreach (QVideoFrame::PixelFormat format, formats) {
+    for (QVideoFrame::PixelFormat format : formats) {
         int index = indexOfYuvColor(format);
 
         if (index != -1) {
@@ -1212,6 +1227,7 @@ void QGstUtils::setMetaData(GstElement *element, const QMap<QByteArray, QVariant
                     tagValue.toDouble(),
                     NULL);
                 break;
+#if GST_CHECK_VERSION(0, 10, 31)
             case QVariant::DateTime: {
                 QDateTime date = tagValue.toDateTime().toLocalTime();
                 gst_tag_setter_add_tags(GST_TAG_SETTER(element),
@@ -1223,6 +1239,7 @@ void QGstUtils::setMetaData(GstElement *element, const QMap<QByteArray, QVariant
                     NULL);
                 break;
             }
+#endif
             default:
                 break;
         }
@@ -1408,8 +1425,10 @@ GstCaps *qt_gst_pad_get_caps(GstPad *pad)
 {
 #if GST_CHECK_VERSION(1,0,0)
     return gst_pad_query_caps(pad, NULL);
-#else
+#elif GST_CHECK_VERSION(0, 10, 26)
     return gst_pad_get_caps_reffed(pad);
+#else
+    return gst_pad_get_caps(pad);
 #endif
 }
 
@@ -1464,6 +1483,76 @@ const gchar *qt_gst_element_get_factory_name(GstElement *element)
     return name;
 }
 
+gboolean qt_gst_caps_can_intersect(const GstCaps * caps1, const GstCaps * caps2)
+{
+#if GST_CHECK_VERSION(0, 10, 25)
+    return gst_caps_can_intersect(caps1, caps2);
+#else
+    GstCaps *intersection = gst_caps_intersect(caps1, caps2);
+    gboolean res = !gst_caps_is_empty(intersection);
+    gst_caps_unref(intersection);
+    return res;
+#endif
+}
+
+#if !GST_CHECK_VERSION(0, 10, 31)
+static gboolean qt_gst_videosink_factory_filter(GstPluginFeature *feature, gpointer)
+{
+  guint rank;
+  const gchar *klass;
+
+  if (!GST_IS_ELEMENT_FACTORY(feature))
+    return FALSE;
+
+  klass = gst_element_factory_get_klass(GST_ELEMENT_FACTORY(feature));
+  if (!(strstr(klass, "Sink") && strstr(klass, "Video")))
+    return FALSE;
+
+  rank = gst_plugin_feature_get_rank(feature);
+  if (rank < GST_RANK_MARGINAL)
+    return FALSE;
+
+  return TRUE;
+}
+
+static gint qt_gst_compare_ranks(GstPluginFeature *f1, GstPluginFeature *f2)
+{
+  gint diff;
+
+  diff = gst_plugin_feature_get_rank(f2) - gst_plugin_feature_get_rank(f1);
+  if (diff != 0)
+    return diff;
+
+  return strcmp(gst_plugin_feature_get_name(f2), gst_plugin_feature_get_name (f1));
+}
+#endif
+
+GList *qt_gst_video_sinks()
+{
+    GList *list = NULL;
+
+#if GST_CHECK_VERSION(0, 10, 31)
+    list = gst_element_factory_list_get_elements(GST_ELEMENT_FACTORY_TYPE_SINK | GST_ELEMENT_FACTORY_TYPE_MEDIA_VIDEO,
+                                                 GST_RANK_MARGINAL);
+#else
+    list = gst_registry_feature_filter(gst_registry_get_default(),
+                                       (GstPluginFeatureFilter)qt_gst_videosink_factory_filter,
+                                       FALSE, NULL);
+    list = g_list_sort(list, (GCompareFunc)qt_gst_compare_ranks);
+#endif
+
+    return list;
+}
+
+void qt_gst_util_double_to_fraction(gdouble src, gint *dest_n, gint *dest_d)
+{
+#if GST_CHECK_VERSION(0, 10, 26)
+    gst_util_double_to_fraction(src, dest_n, dest_d);
+#else
+    qt_real_to_fraction(src, dest_n, dest_d);
+#endif
+}
+
 QDebug operator <<(QDebug debug, GstCaps *caps)
 {
     if (caps) {
