@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -65,15 +71,17 @@ static void qRegisterCameraMetaTypes()
 
 Q_CONSTRUCTOR_FUNCTION(qRegisterCameraMetaTypes)
 
-Q_DECL_CONSTEXPR static bool qt_sizeLessThan(const QSize &s1, const QSize &s2) Q_DECL_NOTHROW
+static bool qt_sizeLessThan(const QSize &s1, const QSize &s2)
 {
     return (s1.width() * s1.height()) < (s2.width() * s2.height());
 }
 
-Q_DECL_CONSTEXPR static bool qt_frameRateRangeLessThan(const QCamera::FrameRateRange &s1, const QCamera::FrameRateRange &s2) Q_DECL_NOTHROW
+static bool qt_frameRateRangeLessThan(const QCamera::FrameRateRange &s1, const QCamera::FrameRateRange &s2)
 {
-    return qFuzzyCompare(s1.maximumFrameRate, s2.maximumFrameRate) ? (s1.minimumFrameRate < s2.minimumFrameRate)
-                                                                   : (s1.maximumFrameRate < s2.maximumFrameRate);
+    if (s1.maximumFrameRate == s2.maximumFrameRate)
+        return s1.minimumFrameRate < s2.minimumFrameRate;
+
+    return s1.maximumFrameRate < s2.maximumFrameRate;
 }
 
 /*!
@@ -273,7 +281,7 @@ void QCameraPrivate::updateLockStatus()
         lockStatuses << q->lockStatus(QCamera::LockWhiteBalance);
 
 
-    foreach (QCamera::LockStatus currentStatus, lockStatuses) {
+    for (QCamera::LockStatus currentStatus : qAsConst(lockStatuses)) {
         int currentPriority = lockStatusPriority.value(currentStatus, -1);
         if (currentPriority > priority) {
             priority = currentPriority;
@@ -653,8 +661,8 @@ QList<QCameraViewfinderSettings> QCamera::supportedViewfinderSettings(const QCam
         return d->viewfinderSettingsControl2->supportedViewfinderSettings();
 
     QList<QCameraViewfinderSettings> results;
-    QList<QCameraViewfinderSettings> supported = d->viewfinderSettingsControl2->supportedViewfinderSettings();
-    Q_FOREACH (const QCameraViewfinderSettings &s, supported) {
+    const QList<QCameraViewfinderSettings> supported = d->viewfinderSettingsControl2->supportedViewfinderSettings();
+    for (const QCameraViewfinderSettings &s : supported) {
         if ((settings.resolution().isEmpty() || settings.resolution() == s.resolution())
                 && (qFuzzyIsNull(settings.minimumFrameRate()) || qFuzzyCompare((float)settings.minimumFrameRate(), (float)s.minimumFrameRate()))
                 && (qFuzzyIsNull(settings.maximumFrameRate()) || qFuzzyCompare((float)settings.maximumFrameRate(), (float)s.maximumFrameRate()))
@@ -685,8 +693,8 @@ QList<QCameraViewfinderSettings> QCamera::supportedViewfinderSettings(const QCam
 QList<QSize> QCamera::supportedViewfinderResolutions(const QCameraViewfinderSettings &settings) const
 {
     QList<QSize> resolutions;
-    QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
-    Q_FOREACH (const QCameraViewfinderSettings &s, capabilities) {
+    const QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
+    for (const QCameraViewfinderSettings &s : capabilities) {
         if (!resolutions.contains(s.resolution()))
             resolutions.append(s.resolution());
     }
@@ -714,8 +722,8 @@ QList<QSize> QCamera::supportedViewfinderResolutions(const QCameraViewfinderSett
 QList<QCamera::FrameRateRange> QCamera::supportedViewfinderFrameRateRanges(const QCameraViewfinderSettings &settings) const
 {
     QList<QCamera::FrameRateRange> frameRateRanges;
-    QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
-    Q_FOREACH (const QCameraViewfinderSettings &s, capabilities) {
+    const QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
+    for (const QCameraViewfinderSettings &s : capabilities) {
         QCamera::FrameRateRange range(s.minimumFrameRate(), s.maximumFrameRate());
         if (!frameRateRanges.contains(range))
             frameRateRanges.append(range);
@@ -743,8 +751,8 @@ QList<QCamera::FrameRateRange> QCamera::supportedViewfinderFrameRateRanges(const
 QList<QVideoFrame::PixelFormat> QCamera::supportedViewfinderPixelFormats(const QCameraViewfinderSettings &settings) const
 {
     QList<QVideoFrame::PixelFormat> pixelFormats;
-    QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
-    Q_FOREACH (const QCameraViewfinderSettings &s, capabilities) {
+    const QList<QCameraViewfinderSettings> capabilities = supportedViewfinderSettings(settings);
+    for (const QCameraViewfinderSettings &s : capabilities) {
         if (!pixelFormats.contains(s.pixelFormat()))
             pixelFormats.append(s.pixelFormat());
     }
