@@ -1,7 +1,8 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Ruslan Baratov
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,30 +11,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,20 +40,24 @@
 
 #include "androidcamera.h"
 #include "androidsurfacetexture.h"
+#include "androidsurfaceview.h"
 #include "qandroidmultimediautils.h"
 
 #include <qstringlist.h>
 #include <qdebug.h>
-#include <qmutex.h>
 #include <QtCore/private/qjnihelpers_p.h>
 #include <QtCore/qthread.h>
+#include <QtCore/qreadwritelock.h>
+#include <QtCore/qmutex.h>
+#include <QtMultimedia/private/qmemoryvideobuffer_p.h>
 
 QT_BEGIN_NAMESPACE
 
-static jclass g_qtCameraListenerClass = 0;
-static QMutex g_cameraMapMutex;
-typedef QMap<int, AndroidCamera *> CameraMap;
-Q_GLOBAL_STATIC(CameraMap, g_cameraMap)
+static const char QtCameraListenerClassName[] = "org/qtproject/qt5/android/multimedia/QtCameraListener";
+
+typedef QHash<int, AndroidCamera *> CameraMap;
+Q_GLOBAL_STATIC(CameraMap, cameras)
+Q_GLOBAL_STATIC(QReadWriteLock, rwLock)
 
 static inline bool exceptionCheckAndClear(JNIEnv *env)
 {
@@ -96,43 +99,67 @@ static QJNIObjectPrivate rectToArea(const QRect &rect)
 // native method for QtCameraLisener.java
 static void notifyAutoFocusComplete(JNIEnv* , jobject, int id, jboolean success)
 {
-    QMutexLocker locker(&g_cameraMapMutex);
-    AndroidCamera *obj = g_cameraMap->value(id, 0);
-    if (obj)
-        Q_EMIT obj->autoFocusComplete(success);
+    QReadLocker locker(rwLock);
+    const auto it = cameras->constFind(id);
+    if (Q_UNLIKELY(it == cameras->cend()))
+        return;
+
+    Q_EMIT (*it)->autoFocusComplete(success);
 }
 
 static void notifyPictureExposed(JNIEnv* , jobject, int id)
 {
-    QMutexLocker locker(&g_cameraMapMutex);
-    AndroidCamera *obj = g_cameraMap->value(id, 0);
-    if (obj)
-        Q_EMIT obj->pictureExposed();
+    QReadLocker locker(rwLock);
+    const auto it = cameras->constFind(id);
+    if (Q_UNLIKELY(it == cameras->cend()))
+        return;
+
+    Q_EMIT (*it)->pictureExposed();
 }
 
 static void notifyPictureCaptured(JNIEnv *env, jobject, int id, jbyteArray data)
 {
-    QMutexLocker locker(&g_cameraMapMutex);
-    AndroidCamera *obj = g_cameraMap->value(id, 0);
-    if (obj) {
-        const int arrayLength = env->GetArrayLength(data);
-        QByteArray bytes(arrayLength, Qt::Uninitialized);
-        env->GetByteArrayRegion(data, 0, arrayLength, (jbyte*)bytes.data());
-        Q_EMIT obj->pictureCaptured(bytes);
-    }
+    QReadLocker locker(rwLock);
+    const auto it = cameras->constFind(id);
+    if (Q_UNLIKELY(it == cameras->cend()))
+        return;
+
+    const int arrayLength = env->GetArrayLength(data);
+    QByteArray bytes(arrayLength, Qt::Uninitialized);
+    env->GetByteArrayRegion(data, 0, arrayLength, (jbyte*)bytes.data());
+    Q_EMIT (*it)->pictureCaptured(bytes);
 }
 
-static void notifyFrameFetched(JNIEnv *env, jobject, int id, jbyteArray data)
+static void notifyNewPreviewFrame(JNIEnv *env, jobject, int id, jbyteArray data,
+                                  int width, int height, int format, int bpl)
 {
-    QMutexLocker locker(&g_cameraMapMutex);
-    AndroidCamera *obj = g_cameraMap->value(id, 0);
-    if (obj) {
-        const int arrayLength = env->GetArrayLength(data);
-        QByteArray bytes(arrayLength, Qt::Uninitialized);
-        env->GetByteArrayRegion(data, 0, arrayLength, (jbyte*)bytes.data());
+    QReadLocker locker(rwLock);
+    const auto it = cameras->constFind(id);
+    if (Q_UNLIKELY(it == cameras->cend()))
+        return;
 
-        Q_EMIT obj->frameFetched(bytes);
-    }
+    const int arrayLength = env->GetArrayLength(data);
+    if (arrayLength == 0)
+        return;
+
+    QByteArray bytes(arrayLength, Qt::Uninitialized);
+    env->GetByteArrayRegion(data, 0, arrayLength, (jbyte*)bytes.data());
+
+    QVideoFrame frame(new QMemoryVideoBuffer(bytes, bpl),
+                      QSize(width, height),
+                      qt_pixelFormatFromAndroidImageFormat(AndroidCamera::ImageFormat(format)));
+
+    Q_EMIT (*it)->newPreviewFrame(frame);
+}
+
+static void notifyFrameAvailable(JNIEnv *, jobject, int id)
+{
+    QReadLocker locker(rwLock);
+    const auto it = cameras->constFind(id);
+    if (Q_UNLIKELY(it == cameras->cend()))
+        return;
+
+    (*it)->fetchLastPreviewFrame();
 }
 
 class AndroidCameraPrivate : public QObject
@@ -155,12 +182,20 @@ public:
     Q_INVOKABLE QSize getPreferredPreviewSizeForVideo();
     Q_INVOKABLE QList<QSize> getSupportedPreviewSizes();
 
+    Q_INVOKABLE QList<AndroidCamera::FpsRange> getSupportedPreviewFpsRange();
+
+    Q_INVOKABLE AndroidCamera::FpsRange getPreviewFpsRange();
+    Q_INVOKABLE void setPreviewFpsRange(int min, int max);
+
     Q_INVOKABLE AndroidCamera::ImageFormat getPreviewFormat();
     Q_INVOKABLE void setPreviewFormat(AndroidCamera::ImageFormat fmt);
+    Q_INVOKABLE QList<AndroidCamera::ImageFormat> getSupportedPreviewFormats();
 
     Q_INVOKABLE QSize previewSize() const { return m_previewSize; }
+    Q_INVOKABLE QSize getPreviewSize();
     Q_INVOKABLE void updatePreviewSize();
     Q_INVOKABLE bool setPreviewTexture(void *surfaceTexture);
+    Q_INVOKABLE bool setPreviewDisplay(void *surfaceHolder);
 
     Q_INVOKABLE bool isZoomSupported();
     Q_INVOKABLE int getMaxZoom();
@@ -212,7 +247,8 @@ public:
 
     Q_INVOKABLE void takePicture();
 
-    Q_INVOKABLE void fetchEachFrame(bool fetch);
+    Q_INVOKABLE void setupPreviewFrameCallback();
+    Q_INVOKABLE void notifyNewFrames(bool notify);
     Q_INVOKABLE void fetchLastPreviewFrame();
 
     Q_INVOKABLE void applyParameters();
@@ -231,13 +267,16 @@ public:
 Q_SIGNALS:
     void previewSizeChanged();
     void previewStarted();
+    void previewFailedToStart();
     void previewStopped();
 
     void autoFocusStarted();
 
     void whiteBalanceChanged();
 
-    void previewFetched(const QByteArray &preview);
+    void takePictureFailed();
+
+    void lastPreviewFrameFetched(const QVideoFrame &frame);
 };
 
 AndroidCamera::AndroidCamera(AndroidCameraPrivate *d, QThread *worker)
@@ -249,25 +288,27 @@ AndroidCamera::AndroidCamera(AndroidCameraPrivate *d, QThread *worker)
     qRegisterMetaType<QList<int> >();
     qRegisterMetaType<QList<QSize> >();
     qRegisterMetaType<QList<QRect> >();
+    qRegisterMetaType<ImageFormat>();
 
     connect(d, &AndroidCameraPrivate::previewSizeChanged, this, &AndroidCamera::previewSizeChanged);
     connect(d, &AndroidCameraPrivate::previewStarted, this, &AndroidCamera::previewStarted);
+    connect(d, &AndroidCameraPrivate::previewFailedToStart, this, &AndroidCamera::previewFailedToStart);
     connect(d, &AndroidCameraPrivate::previewStopped, this, &AndroidCamera::previewStopped);
     connect(d, &AndroidCameraPrivate::autoFocusStarted, this, &AndroidCamera::autoFocusStarted);
     connect(d, &AndroidCameraPrivate::whiteBalanceChanged, this, &AndroidCamera::whiteBalanceChanged);
-    connect(d, &AndroidCameraPrivate::previewFetched, this, &AndroidCamera::previewFetched);
+    connect(d, &AndroidCameraPrivate::takePictureFailed, this, &AndroidCamera::takePictureFailed);
+    connect(d, &AndroidCameraPrivate::lastPreviewFrameFetched, this, &AndroidCamera::lastPreviewFrameFetched);
 }
 
 AndroidCamera::~AndroidCamera()
 {
     Q_D(AndroidCamera);
     if (d->m_camera.isValid()) {
-        g_cameraMapMutex.lock();
-        g_cameraMap->remove(d->m_cameraId);
-        g_cameraMapMutex.unlock();
+        release();
+        QWriteLocker locker(rwLock);
+        cameras->remove(cameraId());
     }
 
-    release();
     m_worker->exit();
     m_worker->wait(5000);
 }
@@ -284,15 +325,14 @@ AndroidCamera *AndroidCamera::open(int cameraId)
     if (!ok) {
         worker->quit();
         worker->wait(5000);
-        delete d;
         delete worker;
         return 0;
     }
 
     AndroidCamera *q = new AndroidCamera(d, worker);
-    g_cameraMapMutex.lock();
-    g_cameraMap->insert(cameraId, q);
-    g_cameraMapMutex.unlock();
+    QWriteLocker locker(rwLock);
+    cameras->insert(cameraId, q);
+
     return q;
 }
 
@@ -356,6 +396,24 @@ QList<QSize> AndroidCamera::getSupportedPreviewSizes()
     return d->getSupportedPreviewSizes();
 }
 
+QList<AndroidCamera::FpsRange> AndroidCamera::getSupportedPreviewFpsRange()
+{
+    Q_D(AndroidCamera);
+    return d->getSupportedPreviewFpsRange();
+}
+
+AndroidCamera::FpsRange AndroidCamera::getPreviewFpsRange()
+{
+    Q_D(AndroidCamera);
+    return d->getPreviewFpsRange();
+}
+
+void AndroidCamera::setPreviewFpsRange(FpsRange range)
+{
+    Q_D(AndroidCamera);
+    QMetaObject::invokeMethod(d, "setPreviewFpsRange", Q_ARG(int, range.min), Q_ARG(int, range.max));
+}
+
 AndroidCamera::ImageFormat AndroidCamera::getPreviewFormat()
 {
     Q_D(AndroidCamera);
@@ -368,12 +426,24 @@ void AndroidCamera::setPreviewFormat(ImageFormat fmt)
     QMetaObject::invokeMethod(d, "setPreviewFormat", Q_ARG(AndroidCamera::ImageFormat, fmt));
 }
 
+QList<AndroidCamera::ImageFormat> AndroidCamera::getSupportedPreviewFormats()
+{
+    Q_D(AndroidCamera);
+    return d->getSupportedPreviewFormats();
+}
+
 QSize AndroidCamera::previewSize() const
 {
     Q_D(const AndroidCamera);
     return d->m_previewSize;
 }
 
+QSize AndroidCamera::actualPreviewSize()
+{
+    Q_D(AndroidCamera);
+    return d->getPreviewSize();
+}
+
 void AndroidCamera::setPreviewSize(const QSize &size)
 {
     Q_D(AndroidCamera);
@@ -399,6 +469,18 @@ bool AndroidCamera::setPreviewTexture(AndroidSurfaceTexture *surfaceTexture)
     return ok;
 }
 
+bool AndroidCamera::setPreviewDisplay(AndroidSurfaceHolder *surfaceHolder)
+{
+    Q_D(AndroidCamera);
+    bool ok = true;
+    QMetaObject::invokeMethod(d,
+                              "setPreviewDisplay",
+                              Qt::BlockingQueuedConnection,
+                              Q_RETURN_ARG(bool, ok),
+                              Q_ARG(void *, surfaceHolder ? surfaceHolder->surfaceHolder() : 0));
+    return ok;
+}
+
 bool AndroidCamera::isZoomSupported()
 {
     Q_D(AndroidCamera);
@@ -641,10 +723,16 @@ void AndroidCamera::takePicture()
     QMetaObject::invokeMethod(d, "takePicture", Qt::BlockingQueuedConnection);
 }
 
-void AndroidCamera::fetchEachFrame(bool fetch)
+void AndroidCamera::setupPreviewFrameCallback()
 {
     Q_D(AndroidCamera);
-    QMetaObject::invokeMethod(d, "fetchEachFrame", Q_ARG(bool, fetch));
+    QMetaObject::invokeMethod(d, "setupPreviewFrameCallback");
+}
+
+void AndroidCamera::notifyNewFrames(bool notify)
+{
+    Q_D(AndroidCamera);
+    QMetaObject::invokeMethod(d, "notifyNewFrames", Q_ARG(bool, notify));
 }
 
 void AndroidCamera::fetchLastPreviewFrame()
@@ -707,6 +795,12 @@ void AndroidCamera::stopPreview()
     QMetaObject::invokeMethod(d, "stopPreview");
 }
 
+void AndroidCamera::stopPreviewSynchronous()
+{
+    Q_D(AndroidCamera);
+    QMetaObject::invokeMethod(d, "stopPreview", Qt::BlockingQueuedConnection);
+}
+
 AndroidCameraPrivate::AndroidCameraPrivate()
     : QObject(),
       m_parametersMutex(QMutex::Recursive)
@@ -728,7 +822,7 @@ bool AndroidCameraPrivate::init(int cameraId)
     if (exceptionCheckAndClear(env) || !m_camera.isValid())
         return false;
 
-    m_cameraListener = QJNIObjectPrivate(g_qtCameraListenerClass, "(I)V", m_cameraId);
+    m_cameraListener = QJNIObjectPrivate(QtCameraListenerClassName, "(I)V", m_cameraId);
     m_info = QJNIObjectPrivate("android/hardware/Camera$CameraInfo");
     m_camera.callStaticMethod<void>("android/hardware/Camera",
                                     "getCameraInfo",
@@ -794,6 +888,9 @@ QSize AndroidCameraPrivate::getPreferredPreviewSizeForVideo()
     QJNIObjectPrivate size = m_parameters.callObjectMethod("getPreferredPreviewSizeForVideo",
                                                            "()Landroid/hardware/Camera$Size;");
 
+    if (!size.isValid())
+        return QSize();
+
     return QSize(size.getField<jint>("width"), size.getField<jint>("height"));
 }
 
@@ -820,12 +917,86 @@ QList<QSize> AndroidCameraPrivate::getSupportedPreviewSizes()
     return list;
 }
 
+QList<AndroidCamera::FpsRange> AndroidCameraPrivate::getSupportedPreviewFpsRange()
+{
+    QMutexLocker parametersLocker(&m_parametersMutex);
+
+    QJNIEnvironmentPrivate env;
+
+    QList<AndroidCamera::FpsRange> rangeList;
+
+    if (m_parameters.isValid()) {
+        QJNIObjectPrivate rangeListNative = m_parameters.callObjectMethod("getSupportedPreviewFpsRange",
+                                                                          "()Ljava/util/List;");
+        int count = rangeListNative.callMethod<jint>("size");
+
+        rangeList.reserve(count);
+
+        for (int i = 0; i < count; ++i) {
+            QJNIObjectPrivate range = rangeListNative.callObjectMethod("get",
+                                                                       "(I)Ljava/lang/Object;",
+                                                                       i);
+
+            jintArray jRange = static_cast<jintArray>(range.object());
+            jint* rangeArray = env->GetIntArrayElements(jRange, 0);
+
+            AndroidCamera::FpsRange fpsRange;
+
+            fpsRange.min = rangeArray[0];
+            fpsRange.max = rangeArray[1];
+
+            env->ReleaseIntArrayElements(jRange, rangeArray, 0);
+
+            rangeList << fpsRange;
+        }
+    }
+
+    return rangeList;
+}
+
+AndroidCamera::FpsRange AndroidCameraPrivate::getPreviewFpsRange()
+{
+    QMutexLocker parametersLocker(&m_parametersMutex);
+
+    QJNIEnvironmentPrivate env;
+
+    AndroidCamera::FpsRange range;
+
+    if (!m_parameters.isValid())
+        return range;
+
+    jintArray jRangeArray = env->NewIntArray(2);
+    m_parameters.callMethod<void>("getPreviewFpsRange", "([I)V", jRangeArray);
+
+    jint* jRangeElements = env->GetIntArrayElements(jRangeArray, 0);
+
+    range.min = jRangeElements[0];
+    range.max = jRangeElements[1];
+
+    env->ReleaseIntArrayElements(jRangeArray, jRangeElements, 0);
+    env->DeleteLocalRef(jRangeArray);
+
+    return range;
+}
+
+void AndroidCameraPrivate::setPreviewFpsRange(int min, int max)
+{
+    QMutexLocker parametersLocker(&m_parametersMutex);
+
+    if (!m_parameters.isValid())
+        return;
+
+    QJNIEnvironmentPrivate env;
+    m_parameters.callMethod<void>("setPreviewFpsRange", "(II)V", min, max);
+    exceptionCheckAndClear(env);
+}
+
 AndroidCamera::ImageFormat AndroidCameraPrivate::getPreviewFormat()
 {
     QMutexLocker parametersLocker(&m_parametersMutex);
 
     if (!m_parameters.isValid())
-        return AndroidCamera::Unknown;
+        return AndroidCamera::UnknownImageFormat;
 
     return AndroidCamera::ImageFormat(m_parameters.callMethod<jint>("getPreviewFormat"));
 }
@@ -841,6 +1012,43 @@ void AndroidCameraPrivate::setPreviewFormat(AndroidCamera::ImageFormat fmt)
     applyParameters();
 }
 
+QList<AndroidCamera::ImageFormat> AndroidCameraPrivate::getSupportedPreviewFormats()
+{
+    QList<AndroidCamera::ImageFormat> list;
+
+    QMutexLocker parametersLocker(&m_parametersMutex);
+
+    if (m_parameters.isValid()) {
+        QJNIObjectPrivate formatList = m_parameters.callObjectMethod("getSupportedPreviewFormats",
+                                                                     "()Ljava/util/List;");
+        int count = formatList.callMethod<jint>("size");
+        for (int i = 0; i < count; ++i) {
+            QJNIObjectPrivate format = formatList.callObjectMethod("get",
+                                                                   "(I)Ljava/lang/Object;",
+                                                                   i);
+            list.append(AndroidCamera::ImageFormat(format.callMethod<jint>("intValue")));
+        }
+    }
+
+    return list;
+}
+
+QSize AndroidCameraPrivate::getPreviewSize()
+{
+    QMutexLocker parametersLocker(&m_parametersMutex);
+
+    if (!m_parameters.isValid())
+        return QSize();
+
+    QJNIObjectPrivate size = m_parameters.callObjectMethod("getPreviewSize",
+                                                           "()Landroid/hardware/Camera$Size;");
+
+    if (!size.isValid())
+        return QSize();
+
+    return QSize(size.getField<jint>("width"), size.getField<jint>("height"));
+}
+
 void AndroidCameraPrivate::updatePreviewSize()
 {
     QMutexLocker parametersLocker(&m_parametersMutex);
@@ -862,6 +1070,15 @@ bool AndroidCameraPrivate::setPreviewTexture(void *surfaceTexture)
     return !exceptionCheckAndClear(env);
 }
 
+bool AndroidCameraPrivate::setPreviewDisplay(void *surfaceHolder)
+{
+    QJNIEnvironmentPrivate env;
+    m_camera.callMethod<void>("setPreviewDisplay",
+                              "(Landroid/view/SurfaceHolder;)V",
+                              static_cast<jobject>(surfaceHolder));
+    return !exceptionCheckAndClear(env);
+}
+
 bool AndroidCameraPrivate::isZoomSupported()
 {
     QMutexLocker parametersLocker(&m_parametersMutex);
@@ -1055,15 +1272,21 @@ void AndroidCameraPrivate::setFocusAreas(const QList<QRect> &areas)
 
 void AndroidCameraPrivate::autoFocus()
 {
+    QJNIEnvironmentPrivate env;
+
     m_camera.callMethod<void>("autoFocus",
                               "(Landroid/hardware/Camera$AutoFocusCallback;)V",
                               m_cameraListener.object());
-    emit autoFocusStarted();
+
+    if (!exceptionCheckAndClear(env))
+        emit autoFocusStarted();
 }
 
 void AndroidCameraPrivate::cancelAutoFocus()
 {
+    QJNIEnvironmentPrivate env;
     m_camera.callMethod<void>("cancelAutoFocus");
+    exceptionCheckAndClear(env);
 }
 
 bool AndroidCameraPrivate::isAutoExposureLockSupported()
@@ -1312,59 +1535,91 @@ void AndroidCameraPrivate::setJpegQuality(int quality)
 
 void AndroidCameraPrivate::startPreview()
 {
-    //We need to clear preview buffers queue here, but there is no method to do it
-    //Though just resetting preview callback do the trick
-    m_camera.callMethod<void>("setPreviewCallbackWithBuffer",
-                              "(Landroid/hardware/Camera$PreviewCallback;)V",
-                              jobject(0));
-    m_cameraListener.callMethod<void>("preparePreviewBuffer", "(Landroid/hardware/Camera;)V", m_camera.object());
-    QJNIObjectPrivate buffer = m_cameraListener.callObjectMethod<jbyteArray>("callbackBuffer");
-    m_camera.callMethod<void>("addCallbackBuffer", "([B)V", buffer.object());
-    m_camera.callMethod<void>("setPreviewCallbackWithBuffer",
-                              "(Landroid/hardware/Camera$PreviewCallback;)V",
-                              m_cameraListener.object());
+    QJNIEnvironmentPrivate env;
+
+    setupPreviewFrameCallback();
     m_camera.callMethod<void>("startPreview");
-    emit previewStarted();
+
+    if (exceptionCheckAndClear(env))
+        emit previewFailedToStart();
+    else
+        emit previewStarted();
 }
 
 void AndroidCameraPrivate::stopPreview()
 {
+    QJNIEnvironmentPrivate env;
+
+    // cancel any pending new frame notification
+    m_cameraListener.callMethod<void>("notifyWhenFrameAvailable", "(Z)V", false);
+
     m_camera.callMethod<void>("stopPreview");
+
+    exceptionCheckAndClear(env);
     emit previewStopped();
 }
 
 void AndroidCameraPrivate::takePicture()
 {
+    QJNIEnvironmentPrivate env;
+
+    // We must clear the preview callback before calling takePicture(), otherwise the call will
+    // block and the camera server will be frozen until the next device restart...
+    // That problem only happens on some devices and on the emulator
+    m_cameraListener.callMethod<void>("clearPreviewCallback", "(Landroid/hardware/Camera;)V", m_camera.object());
+
     m_camera.callMethod<void>("takePicture", "(Landroid/hardware/Camera$ShutterCallback;"
                                              "Landroid/hardware/Camera$PictureCallback;"
                                              "Landroid/hardware/Camera$PictureCallback;)V",
                                               m_cameraListener.object(),
                                               jobject(0),
                                               m_cameraListener.object());
+
+    if (exceptionCheckAndClear(env))
+        emit takePictureFailed();
 }
 
-void AndroidCameraPrivate::fetchEachFrame(bool fetch)
+void AndroidCameraPrivate::setupPreviewFrameCallback()
 {
-    m_cameraListener.callMethod<void>("fetchEachFrame", "(Z)V", fetch);
+    m_cameraListener.callMethod<void>("setupPreviewCallback", "(Landroid/hardware/Camera;)V", m_camera.object());
+}
+
+void AndroidCameraPrivate::notifyNewFrames(bool notify)
+{
+    m_cameraListener.callMethod<void>("notifyNewFrames", "(Z)V", notify);
 }
 
 void AndroidCameraPrivate::fetchLastPreviewFrame()
 {
     QJNIEnvironmentPrivate env;
-    QJNIObjectPrivate data = m_cameraListener.callObjectMethod("lockAndFetchPreviewBuffer", "()[B");
+    QJNIObjectPrivate data = m_cameraListener.callObjectMethod("lastPreviewBuffer", "()[B");
+
     if (!data.isValid()) {
-        m_cameraListener.callMethod<void>("unlockPreviewBuffer");
+        // If there's no buffer received yet, retry when the next one arrives
+        m_cameraListener.callMethod<void>("notifyWhenFrameAvailable", "(Z)V", true);
         return;
     }
+
     const int arrayLength = env->GetArrayLength(static_cast<jbyteArray>(data.object()));
+    if (arrayLength == 0)
+        return;
+
     QByteArray bytes(arrayLength, Qt::Uninitialized);
     env->GetByteArrayRegion(static_cast<jbyteArray>(data.object()),
                             0,
                             arrayLength,
                             reinterpret_cast<jbyte *>(bytes.data()));
-    m_cameraListener.callMethod<void>("unlockPreviewBuffer");
 
-    emit previewFetched(bytes);
+    const int width = m_cameraListener.callMethod<jint>("previewWidth");
+    const int height = m_cameraListener.callMethod<jint>("previewHeight");
+    const int format = m_cameraListener.callMethod<jint>("previewFormat");
+    const int bpl = m_cameraListener.callMethod<jint>("previewBytesPerLine");
+
+    QVideoFrame frame(new QMemoryVideoBuffer(bytes, bpl),
+                      QSize(width, height),
+                      qt_pixelFormatFromAndroidImageFormat(AndroidCamera::ImageFormat(format)));
+
+    emit lastPreviewFrameFetched(frame);
 }
 
 void AndroidCameraPrivate::applyParameters()
@@ -1400,24 +1655,23 @@ QStringList AndroidCameraPrivate::callParametersStringListMethod(const QByteArra
     return stringList;
 }
 
-static JNINativeMethod methods[] = {
-    {"notifyAutoFocusComplete", "(IZ)V", (void *)notifyAutoFocusComplete},
-    {"notifyPictureExposed", "(I)V", (void *)notifyPictureExposed},
-    {"notifyPictureCaptured", "(I[B)V", (void *)notifyPictureCaptured},
-    {"notifyFrameFetched", "(I[B)V", (void *)notifyFrameFetched}
-};
-
 bool AndroidCamera::initJNI(JNIEnv *env)
 {
-    jclass clazz = env->FindClass("org/qtproject/qt5/android/multimedia/QtCameraListener");
+    jclass clazz = QJNIEnvironmentPrivate::findClass(QtCameraListenerClassName,
+                                                     env);
 
-    if (!exceptionCheckAndClear(env) && clazz) {
-        g_qtCameraListenerClass = static_cast<jclass>(env->NewGlobalRef(clazz));
-        if (env->RegisterNatives(g_qtCameraListenerClass,
-                                 methods,
-                                 sizeof(methods) / sizeof(methods[0])) < 0) {
-            return false;
-        }
+    static const JNINativeMethod methods[] = {
+        {"notifyAutoFocusComplete", "(IZ)V", (void *)notifyAutoFocusComplete},
+        {"notifyPictureExposed", "(I)V", (void *)notifyPictureExposed},
+        {"notifyPictureCaptured", "(I[B)V", (void *)notifyPictureCaptured},
+        {"notifyNewPreviewFrame", "(I[BIIII)V", (void *)notifyNewPreviewFrame},
+        {"notifyFrameAvailable", "(I)V", (void *)notifyFrameAvailable}
+    };
+
+    if (clazz && env->RegisterNatives(clazz,
+                                      methods,
+                                      sizeof(methods) / sizeof(methods[0])) != JNI_OK) {
+        return false;
     }
 
     return true;
