@@ -1,31 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Ruslan Baratov
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,7 +62,6 @@ struct AndroidCameraInfo
     QCamera::Position position;
     int orientation;
 };
-Q_DECLARE_TYPEINFO(AndroidCameraInfo, Q_MOVABLE_TYPE);
 
 class AndroidCamera : public QObject
 {
@@ -78,6 +84,26 @@ public:
         YV12 = 842094169
     };
 
+    // http://developer.android.com/reference/android/hardware/Camera.Parameters.html#getSupportedPreviewFpsRange%28%29
+    // "The values are multiplied by 1000 and represented in integers"
+    struct FpsRange {
+        int min;
+        int max;
+
+        FpsRange(): min(0), max(0) {}
+
+        qreal getMinReal() const { return min / 1000.0; }
+        qreal getMaxReal() const { return max / 1000.0; }
+
+        static FpsRange makeFromQReal(qreal min, qreal max)
+        {
+            FpsRange range;
+            range.min = static_cast<int>(min * 1000.0);
+            range.max = static_cast<int>(max * 1000.0);
+            return range;
+        }
+    };
+
     ~AndroidCamera();
 
     static AndroidCamera *open(int cameraId);
@@ -95,11 +121,17 @@ public:
     QSize getPreferredPreviewSizeForVideo();
     QList<QSize> getSupportedPreviewSizes();
 
+    QList<FpsRange> getSupportedPreviewFpsRange();
+
+    FpsRange getPreviewFpsRange();
+    void setPreviewFpsRange(FpsRange);
+
     ImageFormat getPreviewFormat();
     void setPreviewFormat(ImageFormat fmt);
     QList<ImageFormat> getSupportedPreviewFormats();
 
     QSize previewSize() const;
+    QSize actualPreviewSize();
     void setPreviewSize(const QSize &size);
     bool setPreviewTexture(AndroidSurfaceTexture *surfaceTexture);
     bool setPreviewDisplay(AndroidSurfaceHolder *surfaceHolder);
