@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd and/or its subsidiary(-ies).
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd and/or its subsidiary(-ies).
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -179,7 +185,7 @@ AVFCameraInfo AVFCameraSession::cameraDeviceInfo(const QByteArray &device)
 {
     updateCameraDevices();
 
-    Q_FOREACH (const AVFCameraInfo &info, m_cameraDevices) {
+    for (const AVFCameraInfo &info : qAsConst(m_cameraDevices)) {
         if (info.deviceId == device)
             return info;
     }
@@ -220,7 +226,7 @@ void AVFCameraSession::updateCameraDevices()
         //     the screen when held in portrait ==> 270 degrees clockwise angle
         //   - Front-facing cameras have the top side of the sensor aligned with the left side of
         //     the screen when held in portrait ==> 270 degrees clockwise angle
-        // On Mac OS, the position will always be unspecified and the sensor orientation unknown.
+        // On OS X, the position will always be unspecified and the sensor orientation unknown.
         switch (device.position) {
         case AVCaptureDevicePositionBack:
             info.position = QCamera::BackFace;
@@ -285,10 +291,23 @@ void AVFCameraSession::setState(QCamera::State newState)
         Q_EMIT readyToConfigureConnections();
         m_defaultCodec = 0;
         defaultCodec();
-        applyImageEncoderSettings();
-        applyViewfinderSettings();
+
+        bool activeFormatSet = applyImageEncoderSettings()
+                             | applyViewfinderSettings();
+
         [m_captureSession commitConfiguration];
+
+        if (activeFormatSet) {
+            // According to the doc, the capture device must be locked before
+            // startRunning to prevent the format we set to be overriden by the
+            // session preset.
+            [videoCaptureDevice() lockForConfiguration:nil];
+        }
+
         [m_captureSession startRunning];
+
+        if (activeFormatSet)
+            [videoCaptureDevice() unlockForConfiguration];
     }
 
     if (oldState == QCamera::ActiveState) {
@@ -325,6 +344,17 @@ void AVFCameraSession::processSessionStopped()
     }
 }
 
+void AVFCameraSession::onCaptureModeChanged(QCamera::CaptureModes mode)
+{
+    Q_UNUSED(mode)
+
+    const QCamera::State s = state();
+    if (s == QCamera::LoadedState || s == QCamera::ActiveState) {
+        applyImageEncoderSettings();
+        applyViewfinderSettings();
+    }
+}
+
 void AVFCameraSession::attachVideoInputDevice()
 {
     //Attach video input device:
@@ -333,6 +363,7 @@ void AVFCameraSession::attachVideoInputDevice()
             [m_captureSession removeInput:m_videoInput];
             [m_videoInput release];
             m_videoInput = 0;
+            m_activeCameraInfo = AVFCameraInfo();
         }
 
         AVCaptureDevice *videoDevice = m_service->videoDeviceControl()->createCaptureDevice();
@@ -346,6 +377,7 @@ void AVFCameraSession::attachVideoInputDevice()
             qWarning() << "Failed to create video device input";
         } else {
             if ([m_captureSession canAddInput:m_videoInput]) {
+                m_activeCameraInfo = m_cameraDevices.at(m_service->videoDeviceControl()->selectedDevice());
                 [m_videoInput retain];
                 [m_captureSession addInput:m_videoInput];
             } else {
@@ -355,27 +387,31 @@ void AVFCameraSession::attachVideoInputDevice()
     }
 }
 
-void AVFCameraSession::applyImageEncoderSettings()
+bool AVFCameraSession::applyImageEncoderSettings()
 {
     if (AVFImageEncoderControl *control = m_service->imageEncoderControl())
-        control->applySettings();
+        return control->applySettings();
+
+    return false;
 }
 
-void AVFCameraSession::applyViewfinderSettings()
+bool AVFCameraSession::applyViewfinderSettings()
 {
     if (AVFCameraViewfinderSettingsControl2 *vfControl = m_service->viewfinderSettingsControl2()) {
+        QCamera::CaptureModes currentMode = m_service->cameraControl()->captureMode();
         QCameraViewfinderSettings vfSettings(vfControl->requestedSettings());
-        if (AVFImageEncoderControl *imControl = m_service->imageEncoderControl()) {
-            const QSize imageResolution(imControl->imageSettings().resolution());
-            if (!imageResolution.isNull() && imageResolution.isValid()) {
+        // Viewfinder and image capture solutions must be the same, if an image capture
+        // resolution is set, it takes precedence over the viewfinder resolution.
+        if (currentMode.testFlag(QCamera::CaptureStillImage)) {
+            const QSize imageResolution(m_service->imageEncoderControl()->requestedSettings().resolution());
+            if (!imageResolution.isNull() && imageResolution.isValid())
                 vfSettings.setResolution(imageResolution);
-                vfControl->setViewfinderSettings(vfSettings);
-                return;
-            }
         }
 
-        vfControl->applySettings();
+        return vfControl->applySettings(vfSettings);
     }
+
+    return false;
 }
 
 void AVFCameraSession::addProbe(AVFMediaVideoProbeControl *probe)
@@ -414,6 +450,8 @@ FourCharCode AVFCameraSession::defaultCodec()
 
 void AVFCameraSession::onCameraFrameFetched(const QVideoFrame &frame)
 {
+    Q_EMIT newViewfinderFrame(frame);
+
     m_videoProbesMutex.lock();
     QSet<AVFMediaVideoProbeControl *>::const_iterator i = m_videoProbes.constBegin();
     while (i != m_videoProbes.constEnd()) {
