@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,8 +45,10 @@
 QT_BEGIN_NAMESPACE
 
 CameraBinContainer::CameraBinContainer(QObject *parent)
-    :QMediaContainerControl(parent),
-      m_supportedContainers(QGstCodecsInfo::Muxer)
+    :QMediaContainerControl(parent)
+#ifdef HAVE_GST_ENCODING_PROFILES
+    , m_supportedContainers(QGstCodecsInfo::Muxer)
+#endif
 {
     //extension for containers hard to guess from mimetype
     m_fileExtensions["video/x-matroska"] = "mkv";
@@ -54,12 +62,21 @@ CameraBinContainer::CameraBinContainer(QObject *parent)
 
 QStringList CameraBinContainer::supportedContainers() const
 {
+#ifdef HAVE_GST_ENCODING_PROFILES
     return m_supportedContainers.supportedCodecs();
+#else
+    return QStringList();
+#endif
 }
 
 QString CameraBinContainer::containerDescription(const QString &formatMimeType) const
 {
+#ifdef HAVE_GST_ENCODING_PROFILES
     return m_supportedContainers.codecDescription(formatMimeType);
+#else
+    Q_UNUSED(formatMimeType)
+    return QString();
+#endif
 }
 
 QString CameraBinContainer::containerFormat() const
@@ -69,11 +86,13 @@ QString CameraBinContainer::containerFormat() const
 
 void CameraBinContainer::setContainerFormat(const QString &format)
 {
+#ifdef HAVE_GST_ENCODING_PROFILES
     if (m_format != format) {
         m_format = format;
         m_actualFormat = format;
         emit settingsChanged();
     }
+#endif
 }
 
 QString CameraBinContainer::actualContainerFormat() const
@@ -83,7 +102,9 @@ QString CameraBinContainer::actualContainerFormat() const
 
 void CameraBinContainer::setActualContainerFormat(const QString &containerFormat)
 {
+#ifdef HAVE_GST_ENCODING_PROFILES
     m_actualFormat = containerFormat;
+#endif
 }
 
 void CameraBinContainer::resetActualContainerFormat()
@@ -91,6 +112,8 @@ void CameraBinContainer::resetActualContainerFormat()
     m_actualFormat = m_format;
 }
 
+#ifdef HAVE_GST_ENCODING_PROFILES
+
 GstEncodingContainerProfile *CameraBinContainer::createProfile()
 {
     GstCaps *caps;
@@ -99,13 +122,13 @@ GstEncodingContainerProfile *CameraBinContainer::createProfile()
         return 0;
     } else {
         QString format = m_actualFormat;
-        QStringList supportedFormats = m_supportedContainers.supportedCodecs();
+        const QStringList supportedFormats = m_supportedContainers.supportedCodecs();
 
         //if format is not in the list of supported gstreamer mime types,
         //try to find the mime type with matching extension
         if (!supportedFormats.contains(format)) {
             QString extension = suggestedFileExtension(m_actualFormat);
-            foreach (const QString &formatCandidate, supportedFormats) {
+            for (const QString &formatCandidate : supportedFormats) {
                 if (suggestedFileExtension(formatCandidate) == extension) {
                     format = formatCandidate;
                     break;
@@ -127,6 +150,8 @@ GstEncodingContainerProfile *CameraBinContainer::createProfile()
     return profile;
 }
 
+#endif
+
 /*!
   Suggest file extension for current container mimetype.
  */
