@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -308,7 +314,7 @@ static GstCaps *resolutionToCaps(const QSize &resolution,
     if (frameRate > 0.0) {
         gint numerator;
         gint denominator;
-        gst_util_double_to_fraction(frameRate, &numerator, &denominator);
+        qt_gst_util_double_to_fraction(frameRate, &numerator, &denominator);
 
         gst_caps_set_simple(
                     caps,
@@ -404,7 +410,7 @@ void CameraBinSession::setupCaptureResolution()
 
         if (!qFuzzyIsNull(viewfinderFrameRate)) {
             int n, d;
-            gst_util_double_to_fraction(viewfinderFrameRate, &n, &d);
+            qt_gst_util_double_to_fraction(viewfinderFrameRate, &n, &d);
             g_object_set(G_OBJECT(m_videoSrc), "fps-n", n, NULL);
             g_object_set(G_OBJECT(m_videoSrc), "fps-d", d, NULL);
         }
@@ -496,8 +502,8 @@ GstElement *CameraBinSession::buildCameraSource()
                 const QByteArray envVideoSource = qgetenv("QT_GSTREAMER_CAMERABIN_VIDEOSRC");
 
                 if (!envVideoSource.isEmpty()) {
-                    QList<QByteArray> sources = envVideoSource.split(',');
-                    foreach (const QByteArray &source, sources) {
+                    const QList<QByteArray> sources = envVideoSource.split(',');
+                    for (const QByteArray &source : sources) {
                         QList<QByteArray> keyValue = source.split('=');
                         if (keyValue.count() == 1) {
                             m_videoSrc = gst_element_factory_make(keyValue.at(0), "camera_source");
@@ -514,7 +520,8 @@ GstElement *CameraBinSession::buildCameraSource()
                 if (!m_videoSrc)
                     m_videoSrc = gst_element_factory_make("v4l2src", "camera_source");
 
-                g_object_set(G_OBJECT(m_cameraSrc), "video-source", m_videoSrc, NULL);
+                if (m_videoSrc)
+                    g_object_set(G_OBJECT(m_cameraSrc), "video-source", m_videoSrc, NULL);
             }
 
             if (m_videoSrc)
@@ -798,12 +805,14 @@ void CameraBinSession::start()
 
     m_recorderControl->applySettings();
 
+#ifdef HAVE_GST_ENCODING_PROFILES
     GstEncodingContainerProfile *profile = m_recorderControl->videoProfile();
     g_object_set (G_OBJECT(m_camerabin),
                   "video-profile",
                   profile,
                   NULL);
     gst_encoding_profile_unref(profile);
+#endif
 
     setAudioCaptureCaps();
 
@@ -1065,13 +1074,38 @@ bool CameraBinSession::processBusMessage(const QGstreamerMessage &message)
     return false;
 }
 
+QString CameraBinSession::currentContainerFormat() const
+{
+    if (!m_muxer)
+        return QString();
+
+    QString format;
+
+    if (GstPad *srcPad = gst_element_get_static_pad(m_muxer, "src")) {
+        if (GstCaps *caps = qt_gst_pad_get_caps(srcPad)) {
+            gchar *capsString = gst_caps_to_string(caps);
+            format = QString::fromLatin1(capsString);
+            if (capsString)
+                g_free(capsString);
+            gst_caps_unref(caps);
+        }
+        gst_object_unref(GST_OBJECT(srcPad));
+    }
+
+    return format;
+}
+
 void CameraBinSession::recordVideo()
 {
+    QString format = currentContainerFormat();
+    if (format.isEmpty())
+        format = m_mediaContainerControl->actualContainerFormat();
+
     const QString actualFileName = m_mediaStorageLocation.generateFileName(m_sink.isLocalFile() ? m_sink.toLocalFile()
                                                                                                 : m_sink.toString(),
                                       QMediaStorageLocation::Movies,
                                       QLatin1String("clip_"),
-                                      m_mediaContainerControl->suggestedFileExtension(m_mediaContainerControl->actualContainerFormat()));
+                                      m_mediaContainerControl->suggestedFileExtension(format));
 
     m_recordingActive = true;
     m_actualSink = QUrl::fromLocalFile(actualFileName);
@@ -1336,7 +1370,7 @@ QList<QSize> CameraBinSession::supportedResolutions(QPair<int,int> rate,
     //if the range is continuos, populate is with the common rates
     if (isContinuous && res.size() >= 2) {
         //fill the ragne with common value
-        static QList<QSize> commonSizes =
+        static const QList<QSize> commonSizes =
                 QList<QSize>() << QSize(128, 96)
                                << QSize(160,120)
                                << QSize(176, 144)
@@ -1358,7 +1392,7 @@ QList<QSize> CameraBinSession::supportedResolutions(QPair<int,int> rate,
         QSize maxSize = res.last();
         res.clear();
 
-        foreach (const QSize &candidate, commonSizes) {
+        for (const QSize &candidate : commonSizes) {
             int w = candidate.width();
             int h = candidate.height();
 
@@ -1433,14 +1467,28 @@ void CameraBinSession::elementAdded(GstBin *, GstElement *element, CameraBinSess
         g_signal_connect(G_OBJECT(element), "element-removed",  G_CALLBACK(elementRemoved), session);
     } else if (!factory) {
         // no-op
+#if GST_CHECK_VERSION(0,10,31)
     } else if (gst_element_factory_list_is_type(factory, GST_ELEMENT_FACTORY_TYPE_AUDIO_ENCODER)) {
+#else
+    } else if (strstr(gst_element_factory_get_klass(factory), "Encoder/Audio") != NULL) {
+#endif
         session->m_audioEncoder = element;
 
         session->m_audioEncodeControl->applySettings(element);
+#if GST_CHECK_VERSION(0,10,31)
     } else if (gst_element_factory_list_is_type(factory, GST_ELEMENT_FACTORY_TYPE_VIDEO_ENCODER)) {
+#else
+    } else if (strstr(gst_element_factory_get_klass(factory), "Encoder/Video") != NULL) {
+#endif
         session->m_videoEncoder = element;
 
         session->m_videoEncodeControl->applySettings(element);
+#if GST_CHECK_VERSION(0,10,31)
+    } else if (gst_element_factory_list_is_type(factory, GST_ELEMENT_FACTORY_TYPE_MUXER)) {
+#else
+    } else if (strstr(gst_element_factory_get_klass(factory), "Muxer") != NULL) {
+#endif
+        session->m_muxer = element;
     }
 }
 
@@ -1450,6 +1498,8 @@ void CameraBinSession::elementRemoved(GstBin *, GstElement *element, CameraBinSe
         session->m_audioEncoder = 0;
     else if (element == session->m_videoEncoder)
         session->m_videoEncoder = 0;
+    else if (element == session->m_muxer)
+        session->m_muxer = 0;
 }
 
 QT_END_NAMESPACE
