@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -47,7 +53,6 @@
 #include "qwindowsaudioutils.h"
 #include <QtEndian>
 #include <QtCore/QDataStream>
-#include <private/qaudiohelpers_p.h>
 
 //#define DEBUG_AUDIO 1
 
@@ -67,7 +72,7 @@ QWindowsAudioOutput::QWindowsAudioOutput(const QByteArray &device)
     audioSource = 0;
     pullMode = true;
     finished = false;
-    volumeCache = qreal(1.0);
+    volumeCache = (qreal)1.;
 }
 
 QWindowsAudioOutput::~QWindowsAudioOutput()
@@ -275,6 +280,8 @@ bool QWindowsAudioOutput::open()
     timeStampOpened.restart();
     elapsedTimeOffset = 0;
 
+    setVolume(volumeCache);
+
     errorState = QAudio::NoError;
     if(pullMode) {
         deviceState = QAudio::ActiveState;
@@ -400,11 +407,7 @@ qint64 QWindowsAudioOutput::write( const char *data, qint64 len )
             remain = l;
         else
             remain = period_size;
-
-        if (volumeCache < qreal(1.0))
-            QAudioHelperInternal::qMultiplySamples(volumeCache, settings, p, current->lpData, remain);
-        else
-            memcpy(current->lpData, p, remain);
+        memcpy(current->lpData, p, remain);
 
         l -= remain;
         p += remain;
@@ -592,10 +595,16 @@ QAudio::State QWindowsAudioOutput::state() const
 
 void QWindowsAudioOutput::setVolume(qreal v)
 {
-    if (qFuzzyCompare(volumeCache, v))
+    const qreal normalizedVolume = qBound(qreal(0.0), v, qreal(1.0));
+    if (deviceState != QAudio::ActiveState) {
+        volumeCache = normalizedVolume;
         return;
-
-    volumeCache = qBound(qreal(0), v, qreal(1));
+    }
+    const quint16 scaled = normalizedVolume * 0xFFFF;
+    DWORD vol = MAKELONG(scaled, scaled);
+    MMRESULT res = waveOutSetVolume(hWaveOut, vol);
+    if (res == MMSYSERR_NOERROR)
+        volumeCache = normalizedVolume;
 }
 
 qreal QWindowsAudioOutput::volume() const
