@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -76,7 +71,6 @@ private slots:
     void subsequentPlayback();
     void probes();
     void playlist();
-    void playlistObject();
     void surfaceTest_data();
     void surfaceTest();
     void metadata();
@@ -204,6 +198,10 @@ QMediaContent tst_QMediaPlayerBackend::selectMediaFile(const QStringList& mediaC
 
 void tst_QMediaPlayerBackend::initTestCase()
 {
+    QMediaPlayer player;
+    if (!player.isAvailable())
+        QSKIP("Media player service is not available");
+
     const QString testFileName = QFINDTESTDATA("testdata/test.wav");
     QFileInfo wavFile(testFileName);
 
@@ -995,7 +993,6 @@ void tst_QMediaPlayerBackend::playlist()
     QSignalSpy mediaSpy(&player, SIGNAL(mediaChanged(QMediaContent)));
     QSignalSpy currentMediaSpy(&player, SIGNAL(currentMediaChanged(QMediaContent)));
     QSignalSpy stateSpy(&player, SIGNAL(stateChanged(QMediaPlayer::State)));
-    QSignalSpy mediaStatusSpy(&player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)));
     QSignalSpy errorSpy(&player, SIGNAL(error(QMediaPlayer::Error)));
 
     QFileInfo fileInfo(QFINDTESTDATA("testdata/sample.m3u"));
@@ -1004,8 +1001,8 @@ void tst_QMediaPlayerBackend::playlist()
     player.play();
     QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
 
-    if (player.mediaStatus() == QMediaPlayer::InvalidMedia || mediaSpy.count() == 1)
-        QSKIP("QMediaPlayer does not support loading M3U playlists as QMediaPlaylist");
+    if (player.mediaStatus() == QMediaPlayer::InvalidMedia)
+        QSKIP("Media player does not support M3U playlists");
 
     QCOMPARE(mediaSpy.count(), 2);
     // sample.m3u -> sample.m3u resolved -> test.wav ->
@@ -1018,12 +1015,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 11);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 19); // 6 x (LoadingMedia -> BufferedMedia -> EndOfMedia) + NoMedia
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     player.play();
@@ -1032,12 +1027,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 8);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 19); // 6 x (LoadingMedia -> BufferedMedia -> EndOfMedia) + NoMedia
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Invalid - 1st pass >>>
@@ -1052,12 +1045,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 4);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 3); // LoadingMedia -> InvalidMedia -> NoMedia
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Invalid - 2nd pass >>>
@@ -1069,12 +1060,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 3);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 3); // LoadingMedia -> InvalidMedia -> NoMedia
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Invalid2 - 1st pass >>>
@@ -1089,12 +1078,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 6);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 9); // 3 x LoadingMedia + 2 x (BufferedMedia -> EndOfMedia) + InvalidMedia + NoMedia (not in this order)
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Invalid2 - 2nd pass >>>
@@ -1106,12 +1093,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(currentMediaSpy.count(), 5);
     QCOMPARE(stateSpy.count(), 2);
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 9); // 3 x LoadingMedia + 2 x (BufferedMedia -> EndOfMedia) + InvalidMedia + NoMedia (not in this order)
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Recursive - 1st pass >>>
@@ -1130,13 +1115,10 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(stateSpy.count(), 2);
     // there is one invalid media in the master playlist
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 6); // LoadingMedia -> InvalidMedia -> LoadingMedia -> BufferedMedia
-                                         // -> EndOfMedia -> NoMedia
 
     mediaSpy.clear();
     currentMediaSpy.clear();
     stateSpy.clear();
-    mediaStatusSpy.clear();
     errorSpy.clear();
 
     // <<< Recursive - 2nd pass >>>
@@ -1151,141 +1133,6 @@ void tst_QMediaPlayerBackend::playlist()
     QCOMPARE(stateSpy.count(), 2);
     // there is one invalid media in the master playlist
     QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 6); // LoadingMedia -> InvalidMedia -> LoadingMedia -> BufferedMedia
-                                         // -> EndOfMedia -> NoMedia
-}
-
-void tst_QMediaPlayerBackend::playlistObject()
-{
-    QMediaPlayer player;
-
-    QSignalSpy mediaSpy(&player, SIGNAL(mediaChanged(QMediaContent)));
-    QSignalSpy currentMediaSpy(&player, SIGNAL(currentMediaChanged(QMediaContent)));
-    QSignalSpy stateSpy(&player, SIGNAL(stateChanged(QMediaPlayer::State)));
-    QSignalSpy mediaStatusSpy(&player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)));
-    QSignalSpy errorSpy(&player, SIGNAL(error(QMediaPlayer::Error)));
-
-    // --- empty playlist
-    QMediaPlaylist emptyPlaylist;
-    player.setPlaylist(&emptyPlaylist);
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 1);
-    QCOMPARE(currentMediaSpy.count(), 1); // Empty media
-    QCOMPARE(stateSpy.count(), 0);
-    QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 0);
-
-    mediaSpy.clear();
-    currentMediaSpy.clear();
-    stateSpy.clear();
-    mediaStatusSpy.clear();
-    errorSpy.clear();
-
-    // --- Valid playlist
-    QMediaPlaylist playlist;
-    playlist.addMedia(QUrl::fromLocalFile(QFileInfo(QFINDTESTDATA("testdata/test.wav")).absoluteFilePath()));
-    playlist.addMedia(QUrl::fromLocalFile(QFileInfo(QFINDTESTDATA("testdata/_test.wav")).absoluteFilePath()));
-    player.setPlaylist(&playlist);
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 1);
-    QCOMPARE(currentMediaSpy.count(), 3); // test.wav -> _test.wav -> NoMedia
-    QCOMPARE(stateSpy.count(), 2);
-    QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 7); // 2 x (LoadingMedia -> BufferedMedia -> EndOfMedia) + NoMedia
-
-    mediaSpy.clear();
-    currentMediaSpy.clear();
-    stateSpy.clear();
-    mediaStatusSpy.clear();
-    errorSpy.clear();
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 0);
-    QCOMPARE(currentMediaSpy.count(), 4); // playlist -> test.wav -> _test.wav -> NoMedia
-    QCOMPARE(stateSpy.count(), 2);
-    QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 7); // 2 x (LoadingMedia -> BufferedMedia -> EndOfMedia) + NoMedia
-
-    player.setPlaylist(Q_NULLPTR);
-
-    mediaSpy.clear();
-    currentMediaSpy.clear();
-    stateSpy.clear();
-    mediaStatusSpy.clear();
-    errorSpy.clear();
-
-    // --- Nested playlist
-    QMediaPlaylist nestedPlaylist;
-    nestedPlaylist.addMedia(QUrl::fromLocalFile(QFileInfo(QFINDTESTDATA("testdata/_test.wav")).absoluteFilePath()));
-    nestedPlaylist.addMedia(QUrl::fromLocalFile(QFileInfo(QFINDTESTDATA("testdata/test.wav")).absoluteFilePath()));
-    nestedPlaylist.addMedia(&playlist);
-    player.setPlaylist(&nestedPlaylist);
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 1);
-    QCOMPARE(currentMediaSpy.count(), 6); // _test.wav -> test.wav -> nested playlist
-                                          // -> test.wav -> _test.wav -> NoMedia
-    QCOMPARE(stateSpy.count(), 2);
-    QCOMPARE(errorSpy.count(), 0);
-    QCOMPARE(mediaStatusSpy.count(), 13); // 4 x (LoadingMedia -> BufferedMedia -> EndOfMedia) + NoMedia
-
-    player.setPlaylist(Q_NULLPTR);
-
-    mediaSpy.clear();
-    currentMediaSpy.clear();
-    stateSpy.clear();
-    mediaStatusSpy.clear();
-    errorSpy.clear();
-
-    // --- playlist with invalid media
-    QMediaPlaylist invalidPlaylist;
-    invalidPlaylist.addMedia(QUrl("invalid"));
-    invalidPlaylist.addMedia(QUrl::fromLocalFile(QFileInfo(QFINDTESTDATA("testdata/test.wav")).absoluteFilePath()));
-
-    player.setPlaylist(&invalidPlaylist);
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 1);
-    QCOMPARE(currentMediaSpy.count(), 3); // invalid -> test.wav -> NoMedia
-    QCOMPARE(stateSpy.count(), 2);
-    QCOMPARE(errorSpy.count(), 1);
-    QCOMPARE(mediaStatusSpy.count(), 6); // Loading -> Invalid -> Loading -> Buffered -> EndOfMedia -> NoMedia
-
-    player.setPlaylist(Q_NULLPTR);
-
-    mediaSpy.clear();
-    currentMediaSpy.clear();
-    stateSpy.clear();
-    mediaStatusSpy.clear();
-    errorSpy.clear();
-
-    // --- playlist with only invalid media
-    QMediaPlaylist invalidPlaylist2;
-    invalidPlaylist2.addMedia(QUrl("invalid"));
-    invalidPlaylist2.addMedia(QUrl("invalid2"));
-
-    player.setPlaylist(&invalidPlaylist2);
-
-    player.play();
-    QTRY_COMPARE_WITH_TIMEOUT(player.state(), QMediaPlayer::StoppedState, 10000);
-
-    QCOMPARE(mediaSpy.count(), 1);
-    QCOMPARE(currentMediaSpy.count(), 3); // invalid -> invalid2 -> NoMedia
-    QCOMPARE(stateSpy.count(), 2);
-    QCOMPARE(errorSpy.count(), 2);
-    QCOMPARE(mediaStatusSpy.count(), 5); // Loading -> Invalid -> Loading -> Invalid -> NoMedia
 }
 
 void tst_QMediaPlayerBackend::surfaceTest_data()
