@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -50,6 +37,7 @@
 #if !defined(QT_NO_OPENGL) && !defined(QT_OPENGL_ES_1_CL) && !defined(QT_OPENGL_ES_1)
 #include <QtOpenGL/qgl.h>
 #include <QtOpenGL/qglframebufferobject.h>
+#include <QtGui/qopenglfunctions.h>
 #endif
 
 QT_USE_NAMESPACE
@@ -100,12 +88,12 @@ public:
         : QAbstractVideoBuffer(GLTextureHandle)
         , m_textureId(0)
     {
-        glGenTextures(1, &m_textureId);
+        QOpenGLContext::currentContext()->functions()->glGenTextures(1, &m_textureId);
     }
-    
+
     ~QtTestGLVideoBuffer()
     {
-        glDeleteTextures(1, &m_textureId);
+        QOpenGLContext::currentContext()->functions()->glDeleteTextures(1, &m_textureId);
     }
 
     GLuint textureId() const { return m_textureId; }
@@ -114,7 +102,7 @@ public:
 
     uchar *map(MapMode, int *, int *) { return 0; }
     void unmap() {}
-    MapMode	mapMode() const { return NotMapped; }
+    MapMode mapMode() const { return NotMapped; }
 
 private:
     GLuint m_textureId;
@@ -131,7 +119,7 @@ public:
 
     uchar *map(MapMode, int *, int *) { return 0; }
     void unmap() {}
-    MapMode	mapMode() const { return NotMapped; }
+    MapMode mapMode() const { return NotMapped; }
 };
 
 void tst_QPainterVideoSurface::colors()
@@ -570,7 +558,7 @@ void tst_QPainterVideoSurface::shaderType()
     QPainterVideoSurface surface;
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QCOMPARE(surface.shaderType(), QPainterVideoSurface::NoShaders);
@@ -662,7 +650,7 @@ void tst_QPainterVideoSurface::shaderTypeStarted()
 
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QPainterVideoSurface surface;
@@ -907,7 +895,7 @@ void tst_QPainterVideoSurface::shaderSupportedFormat()
 
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QPainterVideoSurface surface;
@@ -1024,7 +1012,7 @@ void tst_QPainterVideoSurface::shaderPresent()
 
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QPainterVideoSurface surface;
@@ -1155,7 +1143,7 @@ void tst_QPainterVideoSurface::shaderPresentOpaqueFrame()
 
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QPainterVideoSurface surface;
@@ -1204,7 +1192,7 @@ void tst_QPainterVideoSurface::shaderPresentGLFrame()
 
     QGLWidget widget;
     widget.show();
-    QTest::qWaitForWindowShown(&widget);
+    QTest::qWaitForWindowExposed(&widget);
     widget.makeCurrent();
 
     QPainterVideoSurface surface;
@@ -1226,12 +1214,13 @@ void tst_QPainterVideoSurface::shaderPresentGLFrame()
 
     QtTestGLVideoBuffer *buffer = new QtTestGLVideoBuffer;
 
-    glBindTexture(GL_TEXTURE_2D, buffer->textureId());
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 2, 2, 0, GL_RGB, GL_UNSIGNED_BYTE, rgb32ImageData);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+    QOpenGLFunctions *f = QOpenGLContext::currentContext()->functions();
+    f->glBindTexture(GL_TEXTURE_2D, buffer->textureId());
+    f->glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 2, 2, 0, GL_RGB, GL_UNSIGNED_BYTE, rgb32ImageData);
+    f->glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    f->glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    f->glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+    f->glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 
     QVideoFrame frame(buffer, QSize(2, 2), QVideoFrame::Format_RGB32);
 
